""" $Id$
	Author: R. Le Gac
"""

__version__ = "$Revision$"


from gluon.storage import Storage
from gluon.tools import PluginManager

ALL = '_all'


def _init_tool(param, type='dict'):
	""" Return the plugin manager with the parameter param
	initialize as a dictionary, if it does not exist.
	
	If can be a list if the type is equal to list.
	
	"""
	p = PluginManager('dbui')
	if param not in p.dbui:
		if type == 'dict':
			p.dbui[param] = {}
		elif type == 'list':
			p.dbui[param] = []
	return p


def customize_field(tablename, fieldname, **extjs):
	""" Helper tool to customize an existing field by applying the 
	Ext JS configuration parameters.
	
	NOTE: If the xtype is defined as a parameter the default widget
	will be replaced by the new one -- not recommand.
	
	"""
	p = _init_tool('customized_fields')
	if tablename not in p.dbui.customized_fields:
		p.dbui.customized_fields[tablename] = {}
	p.dbui.customized_fields[tablename][fieldname] = extjs	


def customize_form(tablename=None, **extjs):
	""" Helper tool to apply Ext JS configuration parameters on existing form.
	For details see the documentation of Ext.form.formPanel.
	
	The most interesting parameters are labelwidth, labelAlign, 
	labelSeparator, defaults, layout, layoutConfig and width.
	
	If tablename is not define, the parameters will be apply to
	all forms. The order is the following apply first the table
	parameters then the general ones.
	
	"""
	p = _init_tool('customized_forms')
	key = tablename
	if key == None: key = ALL
	p.dbui.customized_forms[key] = extjs
	

def customize_form_with_field_set(tablename, title, fields, **extjs):
	""" Helper tool to customize an existing form with fieldset.
	
			tablename
				name of the tablename

			title
				title of the fieldset

			fields
				list of field name, belonging to the tablename,
				groups in the fieldset.

			extjs
				Additional Ext JS configuration parameters
				apply to the fieldset (See the documentation 
				of Ext.form.FieldSet)
	
	"""
	p = _init_tool('field_sets')
	if tablename not in p.dbui.field_sets:
		p.dbui.field_sets[tablename] = []
	
	di = Storage(title=title, fields=fields, extjs= extjs)
	p.dbui.field_sets[tablename].append(di)


def customize_grid(tablename=None, **extjs):
	""" Helper tool to apply Ext JS configuration parameters on existing grid.
	For details see the documentation of Ext.grid.GridPanel.

	The most interesting parameter is plugins:
	pGridRowEditorContextMenu, pGridPaging, .... 

	If tablename is not define, the parameters will be apply to
	all grids. The order is the following apply first the table
	parameters then the general ones.
	
	"""
	p = _init_tool('customized_grids')
	key = tablename
	if key == None: key = ALL
	p.dbui.customized_grids[key] = extjs


def customize_grid_with_template_column(tablename, header, tpl, ipos=0, hidden=[], **extjs):
	""" Helper tool to add a template column to an existing grid.
	
			tablename
				name of the table associate to the grid
				
			header
				Header of the column in the grid
				
			tpl
				Ext.Template for the column
			
			ipos
				position of the column in the grid
				i.e 0 mean the first column, -1 inserts 
				before the last element
				
			hidden
				List of columns to be hidden in the grid since they
				are aggregate in the column template. Columns are identified
				via their fieldname.

			extjs
				Additional Ext JS configuration parameters
				to be apply on the column 
				(See the documentation of Ext.gid.Column)
				
	"""
	p = _init_tool('template_columns')
	p = _init_tool('hidden_columns')
	
	if tablename not in p.dbui.template_columns:
		p.dbui.template_columns[tablename] = []
		p.dbui.hidden_columns[tablename] = []
	
	di = Storage(header=header, tpl=tpl, ipos=ipos, extjs=extjs)
	p.dbui.template_columns[tablename].append(di)
	
	for el in hidden:
		if el not in p.dbui.hidden_columns[tablename]:
			p.dbui.hidden_columns[tablename].append(el)
		
	
def customize_viewport():
	""" Helper tool to customize the viewport
	
	"""
	p = _init_tool('customized_viewport')


def define_combined_field():
	""" Helper tool to define combined fields in form.
	
	"""
	p = _init_tool('combined_fields')


def define_grid_filter():
	""" Helper tool to define filter associate to grid.
	
	"""
	p = _init_tool('grid_filters')


def define_wizard():
	""" Helper tool to define wizard in form.
	
	"""
	p = _init_tool('wizards')


def hidden_tables(*args):
	""" Helper tools to define a set of tables to be hidden in the viewport.
	
	"""
	p = _init_tool('hidden_tables', 'list')
	p.dbui.hidden_tables.extend(args)
	
	