/**
 * The grid row editor plugin
 * 
 * Add a serie of handler to add, delete, duplicate, update and
 * view the selected row of the grid.
 *
 * Most of the handler raise a window containing a form to edit
 * the selected row and to modify it.
 *
 * This plugin limit the number of selectable row to one.
 *
 * NOTE
 * For each handler the scope has to define properly.
 * The scope defined by the caller is the editor object (plugin)
 *
 *
 * @extends Ext.Window
 * @version $Id$
 * 
 */

Ext.namespace('App.grid');

App.grid.RowEditor = Ext.extend(Ext.Window, {

	/**
	 * @cfg {Object} formModel a dictionary defining the form 
	 * embedded in the window. Provide by the server.
	 */
	formModel: null,
	
	/**
	 * pre-defined properties for the window
	 */
	closeAction: 'hide',
	constrain: true,
	modal: true,
	plain: true,

	/**
	 * private method request by the EXtJs component model
	 */
	initComponent: function(){

		// create the FormPanel embedded in the window
		this.items = [this.formModel];
		
		App.grid.RowEditor.superclass.initComponent.call(this);
	},

	/**
	 * Plugin initialization function
	 *
	 * @param {Object} grid GridPanel   the grid instantiating the plugin
	 */
	init: function(grid){

		this.grid = grid;
		this.formPanel = this.items.first();

		// The grid keeps track of the editor
		grid.editor = this;
		
		// The user can only select one row of the grid
		this.grid.selModel = new Ext.grid.RowSelectionModel({
			singleSelect: true,
		});
		
		// Switch off the autoSave mode of the store
		// action on the store required the used the method save
		// in order to commit the change
		this.grid.store.autoSave = false;
		
		// Add a listener to hide the window, clear the form
		// when the transaction with the database is successful
		this.grid.store.on("save", this.endEdit, this);
		
		// HACK: In version 3.2.1 the data in the record are lost somewhere
		// As a consequence, the values appearing in the grid after an
		// update are empty. This fix correct this issue.
		this.grid.store.on("update", function(store, rec, op){
			if(op == Ext.data.Record.COMMIT && Ext.version >= '3.2.1'){

				var fields = this.formPanel.getForm().items;
				var values = this.formPanel.getForm().getFieldValues(true);
				
				for(item in values){
					rec.data[item] =  values[item];
				}
				
				for (key in fields.map){
					var combo = fields.map[key]; 
					if (combo.xtype== 'xcombobox'){
						var cbrec = combo.findRecord(combo.valueField, combo.getValue());
						rec.data[combo.displayField] = cbrec.get(combo.displayField);
					}
				}
			}
		}, this);
	},

	/**
	 * Private function to add a row in the grid using form values
	 * The scope of this function is the App.grid.RowEditor
	 */
	addRowFromForm: function(){

		if(! this.formPanel.getForm().isValid()){return;}

		// generate a fresh new record
		// update its with values found in the form
		var record = new this.grid.store.recordType();
		this.updateRecord(record);
		
		// insert the new record below the selected one
		// if no record is selected insert at the beginning of the store
		// the later append when the user click the button in the tool bar
        var rec = this.grid.getSelectionModel().getSelected();
		var i = this.grid.store.indexOf(rec);
		this.grid.store.insert(i+1, record);
				
		// save the change in the database
		this.grid.store.save();
	},
	
	/**
	 * Private method to update an existing row using form values
	 * The scope of this function is the App.grid.RowEditor
	 */
	editRowFromForm: function(){
		
		if(! this.formPanel.getForm().isValid()){return;}
		
		// update the selected record with modified values
		var record = this.getSelected();
		this.updateRecord(record);
				
		// save the change in the database
		this.grid.store.save();
	},
	
	/**
	 * Private function call when the transaction with the database
	 * is successful. A transaction correspond to the action add, insert
	 * destroy and save.
	 * 
	 * @param {Object} store
	 * @param {Object} batch
	 * @param {Object} data
	 */
	endEdit: function(store, batch, data){

		// clear the form panel
		this.formPanel.getForm().reset();
		
		// hide the window
		this.hide();
	},
	
	/**
	 * Private method to return the record
	 * associate to the row selected in the grid
	 *
	 * In absence of selected row a warning message is shown
	 * and the function return false.
	 */
	getSelected: function(){

        var rec = this.grid.getSelectionModel().getSelected();

        if (!rec) {
			Ext.MessageBox.alert('Warning', 'Select a row please');
            return false;
        }

		return rec;
	},
	
	/**
	 * Handler to add a new row.
	 * The scope use by the handler is defined by the caller.
	 * It has to be the equal to the plugin object.
	 *
	 */
	onAddRow: function(){

		this.setTitle("Create a new record...");
		this.show();
		
		this.formPanel.showButtonOk();
		this.formPanel.buttonOk.on("click", this.addRowFromForm, this);
	},

	/**
	 * Handler to delete the selected row
	 * The scope use by the handler is defined by the caller
	 * It has to be the equal to the plugin object.
	 *
	 */
	onDeleteRow: function(){

        var record = this.getSelected();
        if (! record) {return}

		this.grid.store.remove(record);
		this.grid.store.save();
	},

	/**
	 * Handler to duplicate the selected row
	 * The scope use by the handler is defined by the caller
	 * It has to be the equal to the plugin object.
	 *
	 */
	onDuplicateRow: function(){

        var record = this.getSelected();
        if (! record) {return}

		this.setTitle("Duplicate the record...");
		this.show();

		this.formPanel.showButtonOk();
		this.formPanel.getForm().loadRecord(record);
		this.formPanel.buttonOk.on("click", this.addRowFromForm, this);
	},

	/**
	 * Handler to edit the selected row.
	 * The scope use by the handler is defined by the caller.
	 * It has to be the equal to the plugin object.
	 *
	 */
	onEditRow: function(){

        var record = this.getSelected();
        if (! record) {return}

		this.setTitle("Update the record...");
		this.show();

		this.formPanel.showButtonUpdate();
		this.formPanel.getForm().loadRecord(record);
		this.formPanel.buttonUpdate.on("click", this.editRowFromForm, this);
	},

	/**
	 * Handler to view the selected row in the form.
	 * The scope use by the handler is defined by the caller.
	 * It has to be the equal to the plugin object.
	 *
	 * Field values are not modifiable.
	 * Interactions with the database are switch off.
	 *
	 */
	onViewRow: function(){

        var record = this.getSelected();
		if (! record) {return}

		this.setTitle("View the record...");
		this.show();

		this.formPanel.showButtonOk();
		this.formPanel.getForm().loadRecord(record);
		this.formPanel.buttonClear.hide();
		this.formPanel.buttonOk.hide();


		this.on("hide", function(){
			this.formPanel.buttonClear.show();
			this.formPanel.buttonOk.show();
		}, this);
		
	},
	
	/**
	 * Private function to update the selected record with the value of the form
	 * @param {Object} record
	 */
	updateRecord: function(record){
		
		var fields = this.formPanel.getForm().items;
		var values = this.formPanel.getForm().getFieldValues(true);

		for(item in values){
			record.set(item, values[item]);
		}
		
		// For foreign key, the record display in the grid contains the
		// valueField as well as the displayField.
		// Previous action update the former but note the later.
		// the next line retrieve the displayField
		for (key in fields.map){
			var combo = fields.map[key]; 
			if (combo.xtype== 'xcombobox'){
				var rec = combo.findRecord(combo.valueField, combo.getValue());
				record.set(combo.displayField, rec.get(combo.displayField));
			}
		}
	}
});

Ext.preg('pgridroweditor', App.grid.RowEditor);