/**
 * A Json store to read/write in a table.
 * Design to work with the class DbSvc on the server side.
 * 
 * MANDATORY PROPERTIES:
 * 		- model
 * 
 * All configuration properties of Ext.data.Store are understood.
 *
 * NOTE: The return data contain the raw value for the foreign keys as well as
 * their pointing field. This characteristic is usefull to for combobox and grid.
 * 
 * @extends Ext.data.JsonStore 
 * @author $Author$
 * @version $Id$
 *
 */
Ext.namespace('App.data');

/**
 * Helper class to addapt the Ext.data.JsonWriter to App.data.JsonStore
 *  
 */
App.data.JsonWriter = Ext.extend(Ext.data.JsonWriter, {
	/**
	 * Pre-defined properties
	 */
	encode: true,
	encodeDelete: true,
	/**
	 * constructor
	 */
	constructor: function(config){
		App.data.JsonWriter.superclass.constructor.call(this, config);
	},
	/**
	 * The render function finalizes the params object. 
	 * In the base class, this method copies the baseParams and the data.
	 * Addapt the base class method to remove the baseParams which are not
	 * used by the server for the write actions
	 * 
	 * @param {Object} params
	 * @param {Object} baseParams
	 * @param {Object} data
	 */
	render: function(params, baseParams, data){
				
		// the baseParams contains many information to customize database actions.
		// Keep only information requires in write transaction 
		var tmp = Ext.apply({}, baseParams);
		for (var el in baseParams){
			if (el != "table" && el != "debug"){
				delete baseParams[el];	
			}
		}
		
		// execute the underlying method
		App.data.JsonWriter.superclass.render.apply(this, arguments);
		
		// restore the baseParams for later use
		Ext.apply(baseParams, tmp);
	}
});

/**
 * HTTP Exception handler
 * @param {Object} dataproxy
 * @param {Object} type
 * @param {Object} action
 * @param {Object} options
 * @param {Object} response
 * @param {Object} args
 */
App.data.httpException = function(dataproxy, type, action, options, response, args){
			
	if(response.status == 200){
		
		console.log('response');
		console.dir(response);
		
		// handle error coming from validation step on the server
		// decode the return json string
		var dict = Ext.decode(response.responseText);
		
		// build the error message
		var text = new String();
		for(item in dict.errors){
			text += "Field <i>"+item+"</i>: "+dict.errors[item]+"<br>";
		}
		text += "The record is not <i>"+action+"</i> in the database."
		
		// popup an alert
		Ext.MessageBox.show({
			title: "DATABASE ERROR",
			msg: text,
			buttons: Ext.MessageBox.OK,
			icon: Ext.MessageBox.ERROR
		});							
	} else {
		// handle all other king of errors
		Ext.MessageBox.show({
			title: response.statusText,
			msg: response.responseText,
			buttons: Ext.MessageBox.OK,
			icon: Ext.MessageBox.ERROR
		});			
	}

};

/**
 * App.data.JsnStore
 */
App.data.JsonStore = Ext.extend(Ext.data.Store, {

	/**
	 * @cfg {Object} model Object to configure the store for a table of the database. 
	 * It is provided by the server and contains the following keys:
	 * 	{
	 * 		table: 'xx',
	 * 		fields: [field1, firld2,...],
	 * 		dbFields: [(table1, field1),...], 
	 * 		where: [...],
	 * 		leftjoin: ?
	 * 		groupby: ?
	 * 		orderby: ?
	 * } 
	 */
	model: null,
	/**
	 * 
	 * @cfg {Boolean} debug (optional) avtivate the debug mode 
	 */
	debug: false,
	
	/**
	 * The constructor
	 */	
	constructor: function(config){
	
	
		Ext.apply(this, config);
		
		if (!this.model) {
			throw new Error("the property model is missing !!!")
		}

		// Setup json reader if not defined in config
		// Assume that we are reading data coming from a single table.
		if (!config.reader) {
						
			this.reader = new Ext.data.JsonReader({
				root: 'data',
				fields: this.model.fields,
				totalProperty: 'results',
				idProperty: this.model.idProperty,
			});
		}
		
		// Setup the json writer if not defined in config
		if (!config.writer) {
			this.writer = new App.data.JsonWriter();
		}
		
		// construct the underlying class. DON'T MOVE
		App.data.JsonStore.superclass.constructor.apply(this);
		
		// Setup the baseParams parameters
		// They are always send to the server and allows to steer the action
		// In fact copy the model except the fields property
		for (var el in this.model){
			if (el != "fields") {
				this.baseParams[el] = this.model[el];
			}
		}
	
		// activate the debug mode
		if (this.debug){
			this.baseParams["debug"] = true;
		}			
		
		// add a listener to catch HTTP failure
		this.addListener('exception', App.data.httpException);
	}
});
